@@ -27,9 +27,13 @@
 #include "util.h"
 #include "atmd.h"
 
+#include <unistd.h>		/* gethostname()	*/
+#include <netdb.h>		/* gethostbyname()	*/
+
 #define COMPONENT "MIB"
 
 static Variable variables[] =
+/* keep this sorted by the value of entry.name */
 {
   { &sysDescr, getString, NULL, NULL, &sysDescrValue },
   { &sysObjectID, getOid, NULL, NULL, &sysObjectIDValue },
@@ -39,7 +43,9 @@ static Variable variables[] =
   { &sysLocation, getString, NULL, NULL, &sysLocationValue },
   { &sysServices, getInteger, NULL, NULL, &sysServicesValue },
   { &atmfPortIndex, getInteger, NULL, NULL, &atmfPortIndexValue },
-  { &atmfMyIpNmAddress, getOid, NULL, NULL, &atmfMyIpNmAddressValue },
+  { &atmfMyIpNmAddress, getIpAddr, NULL, NULL, &atmfMyIpNmAddressValue },
+  { &atmfAtmLayerUniVersion, getInteger, NULL, NULL,
+     &atmfAtmLayerUniVersionValue },
   { &atmfNetPrefixStatus, getNetPrefix, getnextNetPrefix, setNetPrefix, NULL },
   { NULL }
 };
@@ -232,6 +238,116 @@ AsnInt getInteger(VarBind *varbind, Variable *var)
 }
 
 
+#include <stdint.h>
+#include <string.h>
+#include <ctype.h>
+
+#include <sys/types.h>
+#include <sys/socket.h>
+#include <sys/ioctl.h>
+#include <net/if.h>
+#include <netinet/in.h>
 
 
+#define MAX_ITFS 500
 
+
+static const char *itf_types[] = {
+    "",					/* fallback */
+    "slip*","ppp*","eth*","atm*",	/* candidates */
+    "!lo",				/* blacklist */
+    NULL				/* the end */
+};
+
+
+static uint32_t get_local_ip(void)
+{
+    struct ifconf ifc;
+    struct ifreq ifr[MAX_ITFS];
+    const char **best;
+    uint32_t best_ip;
+    int s,i;
+
+    s = socket(PF_INET,SOCK_DGRAM,0);
+    if (s < 0) {
+	perror("socket");
+	return 0;
+    }
+    ifc.ifc_len = MAX_ITFS*sizeof(struct ifreq);
+    ifc.ifc_req = ifr;
+    if (ioctl(s,SIOCGIFCONF,&ifc) < 0) {
+	perror("SIOCGIFCONF");
+	return 0;
+    }
+    best = itf_types;
+    best_ip = 0;
+    for (i = 0; i < ifc.ifc_len/sizeof(struct ifreq); i++) {
+	struct sockaddr_in *addr;
+	const char **walk;
+	uint32_t ip;
+
+	addr = (struct sockaddr_in *) &ifr[i].ifr_addr;
+	if (addr->sin_family != AF_INET) continue;
+	ip = addr->sin_addr.s_addr;
+	if (!ip) continue;
+	if (ioctl(s,SIOCGIFFLAGS,&ifr[i]) < 0) {
+	    perror("SIOCGIFFLAGS");
+	    continue;
+	}
+	if (ifr[i].ifr_flags & IFF_LOOPBACK) continue;
+	if (!(ifr[i].ifr_flags & IFF_UP)) continue;
+	for (walk = best+1; *walk; walk++) {
+	    const char *pos,*wc,*end;
+	    int not;
+
+	    if ((not = *(pos = *walk) == '!')) pos++;
+	    if (*pos) {
+		int len;
+
+		wc = strchr(pos,'*');
+		end = wc ? wc : strchr(pos,0);
+		len = end-pos;
+		if (strncmp(ifr[i].ifr_name,pos,end-pos)) continue;
+		if (len < IFNAMSIZ && ifr[i].ifr_name[len])
+		    if (!wc || !isdigit(ifr[i].ifr_name[len])) continue;
+	    }
+	    if (not) break;
+	    best = walk;
+	    best_ip = ip;
+	}
+    }
+    return best_ip;
+}
+
+
+AsnInt getIpAddr(VarBind *varbind, Variable *var)
+{
+  if (!((IpAddress*) var->value)->octs) {/* not set */
+      uint32_t ip;
+      ip = get_local_ip();
+      if (ip) {
+	((IpAddress*) var->value)->octs = alloc(4);
+	*((uint32_t *) ((IpAddress*) var->value)->octs) = ip;
+      }
+  }
+  if (!((IpAddress*) var->value)->octs) /* still not set */
+    {
+      struct hostent* h;
+      char hostname[128];
+      ((IpAddress*) var->value)->octs = alloc(4);
+      if (!gethostname(hostname, sizeof(hostname)-1))
+	{
+	  h = (struct hostent*) gethostbyname(hostname);
+	  *((uint32_t *) ((IpAddress*) var->value)->octs) =
+	    *((uint32_t *) h->h_addr_list[0]);
+	}
+    };
+  varbind->value = Asn1Alloc(sizeof(ObjectSyntax));
+  varbind->value->choiceId = OBJECTSYNTAX_APPLICATION_WIDE;
+  varbind->value->a.application_wide = Asn1Alloc(sizeof(ApplicationSyntax));
+  varbind->value->a.application_wide->choiceId = APPLICATIONSYNTAX_ADDRESS;
+  varbind->value->a.application_wide->a.address = Asn1Alloc(sizeof(NetworkAddress));
+  varbind->value->a.application_wide->a.address->choiceId = NETWORKADDRESS_INTERNET;
+  varbind->value->a.application_wide->a.address->a.internet = (IpAddress*) var->value;
+  return NOERROR;
+}
