@@ -5,12 +5,13 @@
 #include <atm.h>
 #include <linux/atmmpc.h>
 #include <netinet/in.h>
+#include <limits.h> /* For UINT_MAX */
 #include "io.h"
 #include "k_interf.h"
 #include "packets.h"
 
-extern uint8_t MPS_CTRL_ATM_ADDR[]; /* from main.c */
-extern int keep_alive_sm_running;   /* from io.c   */
+extern struct mpc_control mpc_control;  /* from main.c */
+extern int keep_alive_sm_running;       /* from io.c   */
 
 #if 0
 #define dprintf printf
@@ -53,7 +54,7 @@ int parse_extensions(uint8_t *buff,struct extension_values * values ){
         pos += sizeof(uint32_t);
       }
       else
-	pos += sizeof(struct nhrp_extension_with_value);
+	pos += sizeof(struct nhrp_extension);
       break;
     case MPOA_ATM_SERVICE_CATEGORY_EXTENSION:
       printf("mpcd: p_recogn.c: mpoa_atm_service_category_extension.");
@@ -96,6 +97,20 @@ int parse_extensions(uint8_t *buff,struct extension_values * values ){
   return pos;
 }  
 
+static uint32_t calculate_ip_mask(uint8_t prefix){
+  int i;
+  uint32_t mask = 0;
+  if(prefix >= 32)
+    return UINT_MAX;
+  for( i = 0; i < prefix - 1; i++){
+    mask = mask | 1;
+    mask = mask << 1;
+  }
+  mask = mask | 1;
+  mask = mask << (32 - prefix);
+  return mask;
+}
+
 static void print_cie_code(uint8_t code){
   printf("mpcd: p_recogn.c: resolution_reply_rcvd: ");
   switch(code){
@@ -145,6 +160,7 @@ static void print_cie_code(uint8_t code){
 static int nhrp_purge_request(uint8_t *buff){
   int pos = 0;
   int cie_limit = 0;
+  uint32_t ip_mask;
   uint8_t  eg_MPC_data_ATM_addr[ATM_ESA_LEN];
   uint32_t eg_MPS_ip_addr;
   uint32_t purge_ip;
@@ -177,16 +193,19 @@ static int nhrp_purge_request(uint8_t *buff){
     cie = (struct nhrp_cie *)(buff + pos);
     if(cie->cli_addr_tl){
       purge_ip = cie->cli_protocol_address;
+      ip_mask = calculate_ip_mask(cie->prefix_length);
       pos += sizeof(struct nhrp_cie);
     }
     else{
       cie_no_nbma = (struct nhrp_cie_no_nbma *)(buff + pos);
       purge_ip = cie_no_nbma->cli_protocol_address;
+      ip_mask = calculate_ip_mask(cie_no_nbma->prefix_length);
       pos += sizeof(struct nhrp_cie_no_nbma);
     }
     msg.type = INGRESS_PURGE_RCVD;
+    msg.ip_mask = ip_mask;
     msg.content.in_info.in_dst_ip = purge_ip;
-    memcpy(msg.MPS_ctrl, MPS_CTRL_ATM_ADDR, ATM_ESA_LEN);
+    memcpy(msg.MPS_ctrl, mpc_control.MPS_CTRL_ATM_ADDR, ATM_ESA_LEN);
     send_to_kernel(&msg);
   }
   if(ntohs(fixed->ar_extoff)){
@@ -212,6 +231,7 @@ static int mpoa_cache_imposition_request( uint8_t * buff ){
   pos += sizeof(struct nhrp_common_h);
   cie_short = (struct nhrp_cie_short *)(buff + pos);
   msg.content.eg_info.holding_time = ntohs(cie_short->holding_time);
+  msg.ip_mask = calculate_ip_mask(cie_short->prefix_length);
   if(ntohs(cie_short->holding_time))
     keep_alive_sm_running = 1;
   if(ntohs(fixed->ar_extoff)){
@@ -225,7 +245,7 @@ static int mpoa_cache_imposition_request( uint8_t * buff ){
   if(common->dst_proto_len)
     msg.content.eg_info.eg_dst_ip = common->dst_protocol_address;
   msg.content.eg_info.tag = new_tag(msg.content.eg_info.cache_id);
-  memcpy(msg.MPS_ctrl, MPS_CTRL_ATM_ADDR, ATM_ESA_LEN);
+  memcpy(msg.MPS_ctrl, mpc_control.MPS_CTRL_ATM_ADDR, ATM_ESA_LEN);
   msg.type = CACHE_IMPOS_RCVD;
   send_to_kernel(&msg);
   return send_cache_imposition_reply( buff, msg.content.eg_info.tag,
@@ -237,6 +257,7 @@ static int mpoa_egress_cache_purge_reply( uint8_t * buff ){
   int pos = 0;
   struct nhrp_fixed_h *fixed = (struct nhrp_fixed_h *)buff;
   struct nhrp_common_h *common;
+  struct nhrp_cie *cie;
   struct extension_values values;
   struct k_message msg;
   memset(&msg,0,sizeof(struct k_message));
@@ -244,11 +265,14 @@ static int mpoa_egress_cache_purge_reply( uint8_t * buff ){
   common = (struct nhrp_common_h*)(buff + pos);
   if(!check_incoming(ntohl(common->request_ID), MPOA_EGRESS_CACHE_PURGE_REQUEST))
     return -1;
+  pos += sizeof(struct nhrp_common_h);
+  cie = (struct nhrp_cie *)(buff + pos);
+  msg.ip_mask = calculate_ip_mask(cie->prefix_length);
   if(fixed->ar_extoff)
     parse_extensions(buff + ntohs(fixed->ar_extoff),&values);
   msg.content.eg_info.cache_id = values.dll_ext.cache_id;
   msg.type = EGRESS_PURGE_RCVD;
-  memcpy(msg.MPS_ctrl,MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
+  memcpy(msg.MPS_ctrl,mpc_control.MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
   return send_to_kernel(&msg);
 }
 
@@ -261,7 +285,7 @@ static int mpoa_keep_alive(uint8_t *buff){
   pos += sizeof(struct nhrp_fixed_h);
   common = (struct nhrp_common_h_no_ip *)(buff + pos); 
   sequence_nmbr = ntohl(common->request_ID);
-  if(memcmp(common->src_nbma_address,MPS_CTRL_ATM_ADDR,ATM_ESA_LEN)){
+  if(memcmp(common->src_nbma_address,mpc_control.MPS_CTRL_ATM_ADDR,ATM_ESA_LEN)){
     printf("mpcd: p_recogn.c: new MPS! \n" );
     return -1;
   }
@@ -285,7 +309,7 @@ static int mpoa_trigger( uint8_t * buff ){
   if(ntohs(fixed->ar_extoff))
     parse_extensions(buff+ntohs(fixed->ar_extoff),&values);
   msg.content.in_info.in_dst_ip = common->dst_protocol_address;
-  memcpy(msg.MPS_ctrl,MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
+  memcpy(msg.MPS_ctrl,mpc_control.MPS_CTRL_ATM_ADDR,ATM_ESA_LEN);
   msg.type = MPOA_TRIGGER_RCVD;
   send_to_kernel(&msg);
   return 1;
@@ -320,15 +344,34 @@ static int mpoa_resolution_reply( uint8_t * buff ){
   msg.type = MPOA_RES_REPLY_RCVD;
   msg.content.in_info.in_dst_ip  = common->dst_protocol_address;
   memcpy(msg.content.in_info.eg_MPC_ATM_addr,cie->cli_nbma_address,ATM_ESA_LEN);
-  memcpy(msg.MPS_ctrl,MPS_CTRL_ATM_ADDR, ATM_ESA_LEN);
+  memcpy(msg.MPS_ctrl,mpc_control.MPS_CTRL_ATM_ADDR, ATM_ESA_LEN);
   send_to_kernel(&msg);
   keep_alive_sm_running = 1;
   return MPOA_RESOLUTION_REPLY;
 }
 
+
+static int checksum_check(uint8_t *buff){
+  uint16_t checksum;
+  int length;
+  struct nhrp_fixed_h *fixed = (struct nhrp_fixed_h*)buff;
+  checksum = fixed->ar_chksum;
+  fixed->ar_chksum = 0;
+  length = ntohs(fixed->ar_pktsz);
+  if(checksum != compute_ip_csum(buff,length))
+    return 0;
+  return 1;
+}
+
+
 int recognize_packet(uint8_t * buff){
   struct nhrp_fixed_h *fixed = (struct nhrp_fixed_h *)buff;
-  uint8_t  type = fixed->ar_op_type;
+  uint8_t  type;
+  if(!checksum_check(buff)){
+    printf("mpcd: p_recogn.c: checksum error!\n");
+    return -1;
+  }
+  type = fixed->ar_op_type;
   dprintf("mpcd: p_recogn.c: ");
   switch(type){
   case NHRP_PURGE_REQUEST :
@@ -344,7 +387,7 @@ int recognize_packet(uint8_t * buff){
     dprintf("keep alive recieved. \n");
     return mpoa_keep_alive(buff);
   case MPOA_TRIGGER :
-    dprintf("mpoa trigger recieved. \n");
+    printf("mpoa trigger recieved. \n");
     return mpoa_trigger(buff);
   case MPOA_RESOLUTION_REPLY:
     dprintf("mpoa resolution reply recieved. \n");
@@ -354,9 +397,3 @@ int recognize_packet(uint8_t * buff){
     return -1;
   }
 }
-
-
-
-
-
-
