@@ -107,10 +107,10 @@
 #define ATM_CV_UNALLOC		  1 /* unallocated (unassigned) number */
 #define ATM_CV_NO_ROUTE_TNS	  2 /* no route to specified transit network */
 #define ATM_CV_NO_ROUTE_DEST	  3 /* no route to destination */
-#ifdef UNI30
+#if defined(UNI30) || defined(DYNAMIC_UNI)
 #define ATM_CV_CI_UNACC		 10 /* VPCI/VCI unacceptable */
 #endif
-#if defined(UNI31) || defined(UNI40)
+#if defined(UNI31) || defined(UNI40) || defined(DYNAMIC_UNI)
 #define ATM_CV_NORMAL_CLEAR	 16 /* normal call clearing */
 #endif
 #define ATM_CV_USER_BUSY	 17 /* user busy */
@@ -126,9 +126,9 @@
 #define ATM_CV_NORMAL_UNSPEC	 31 /* normal, unspecified */
 /* ----------------------------------- resource unavailable */
 #define ATM_CV_CI_UNAVAIL	 35 /* requested VPCI/VCI unavailable */
-#if defined(UNI31) || defined(UNI40)
+#if defined(UNI31) || defined(UNI40) || defined(DYNAMIC_UNI)
 #define ATM_CV_CI_FAIL		 36 /* VPCI/VCI assignment failure */
-#define ATM_CV_UCR_UNAVAIL	 37 /* user cell rate not available */
+#define ATM_CV_UCR_UNAVAIL_NEW	 37 /* user cell rate not available (>3.0)*/
 #endif
 #define ATM_CV_NET_OOO		 38 /* network out of order - unused */
 #define ATM_CV_TEMP_FAIL	 41 /* temporary failure */
@@ -137,8 +137,8 @@
 #define ATM_CV_RES_UNAVAIL	 47 /* resource unavailable, unspecified */
 /* ----------------------------------- service or option not available */
 #define ATM_CV_QOS_UNAVAIL	 49 /* Quality of Service unavailable */
-#ifdef UNI30
-#define ATM_CV_UCR_UNAVAIL	 51 /* user cell rate not available */
+#if defined(UNI30) || defined(DYNAMIC_UNI)
+#define ATM_CV_UCR_UNAVAIL_OLD	 51 /* user cell rate not available (3.0) */
 #endif
 #define ATM_CV_BBCAP_NOT_AUTH	 57 /* bearer capability not authorized */
 #define ATM_CV_BBCAP_UNAVAIL	 58 /* bearer capability not presently
@@ -149,8 +149,8 @@
 #define ATM_CV_BBCAP_NOT_IMPL	 65 /* bearer capability not implemented */
 #define ATM_CV_UNSUPP_TRAF_PRM	 73 /* unsupported combination of traffic
 				       parameters */
-#if defined(UNI31) || defined(UNI40)
-#define ATM_CV_AAL_UNSUPP	 78 /* AAL parameters cannot be supported */
+#if defined(UNI31) || defined(UNI40) || defined(DYNAMIC_UNI)
+#define ATM_CV_AAL_UNSUPP_NEW	 78 /* AAL param. cannot be supported (>3.0) */
 #endif
 /* ----------------------------------- invalid message */
 #define ATM_CV_INV_CR		 81 /* invalid call reference value */
@@ -159,8 +159,8 @@
 #define ATM_CV_INV_EPR		 89 /* invalid endpoint reference */
 #define ATM_CV_INV_TNS		 91 /* invalid transit network selection */
 #define ATM_CV_TOO_MANY_APR	 92 /* too many pending add party requests */
-#ifdef UNI30
-#define ATM_CV_AAL_UNSUPP	 93 /* AAL parameters cannot be supported */
+#if defined(UNI30) || defined(DYNAMIC_UNI)
+#define ATM_CV_AAL_UNSUPP_OLD	 93 /* AAL param. cannot be supported (3.0) */
 #endif
 /* ----------------------------------- protocol error */
 #define ATM_CV_MAND_IE_MISSING	 96 /* mandatory information element is
@@ -263,7 +263,7 @@
 
 #define ATM_POE_EXC_EXC		0 /* exclusive VPCI; exclusive VCI */
 #define ATM_POE_EXC_ANY		1 /* exclusive VPCI; any VCI */
-#ifdef UNI40
+#if defined(UNI40) || defined(DYNAMIC_UNI)
 #define ATM_POE_EXC_NO		2 /* exclusive VPCI; no VCI (used for VPCs) */
 #endif
 
