@@ -38,6 +38,7 @@ const char *dump_dir = NULL;
 
 SIG_ENTITY _entity = {
 	0,		/* fd */
+	0,		/* unspecified UNI version */
 	sm_user,	/* mode */
 	-1,		/* sig_pcr; obsolete @@@ */
 	NULL		/* sig_qos */
@@ -339,6 +340,7 @@ int main(int argc,char **argv)
 	    diag(COMPONENT,DIAG_FATAL,"Error in config file. - Aborting.");
     if (!atmpvc_addr_in_use(_entity.signaling_pvc))
 	_entity.signaling_pvc.sap_addr.vci = 5;
+    if (!_entity.uni) _entity.uni = S_UNI30;
     /* process all other options but -c */
     optind = 0;
     while ((c = getopt(argc,argv,"Abc:dD:l:m:nNP:q:t:")) != EOF)
@@ -432,23 +434,16 @@ int main(int argc,char **argv)
 	if (chdir(dump_dir) < 0)
 	    diag(COMPONENT,DIAG_ERROR,"chdir %s: %s",dump_dir,strerror(errno));
     for (sig = entities; sig; sig = sig->next) {
-	diag(COMPONENT,DIAG_INFO,"Linux ATM signaling "
-#ifdef UNI30
-	  "UNI 3.0"
+	diag(COMPONENT,DIAG_INFO,"Linux ATM signaling %s "
+#ifdef DYNAMIC_UNI
+	  "(dynamic)"
 #endif
-#ifdef UNI31
-	  "UNI 3.1"
-#ifdef ALLOW_UNI30
-	  "+3.0compat"
-#endif
-#endif
-#ifdef UNI40
-	   "UNI 4.0"
-#ifdef Q2963_1
-	  "+Q.2931.1"
-#endif
-#endif
-          "/AAL5, version " VERSION " on %d.%d.%d",S_PVC(sig));
+	  ", version " VERSION " on %d.%d.%d",
+	  sig->uni & S_UNI30 ?
+	    sig->uni & S_UNI31 ? "UNI 3.1+3.0compat" : "UNI 3.0" :
+	    sig->uni & S_UNI31 ? "UNI 3.1" :
+	      sig->uni & S_Q2963_1 ? "UNI 4.0+Q.2963.1" : "UNI 4.0",
+          S_PVC(sig));
 	diag(COMPONENT,DIAG_INFO,"Acting as %s",
 	  sig->mode == sm_user ? "USER side" :
 	  sig->mode == sm_net ? "NETWORK side" : "SWITCH");
@@ -459,12 +454,7 @@ int main(int argc,char **argv)
     for (sig = entities; sig; sig = sig->next) {
 	if (sig->mode != sm_switch) init_addr(sig);
 	start_saal(&sig->saal,&ops,sig,
-#ifdef UNI30
-	  sscop_qsaal1
-#else
-	  sscop_q2110
-#endif
-	  );
+	  sig->uni == S_UNI30 ? sscop_qsaal1 : sscop_q2110);
 	saal_estab_req(&sig->saal,NULL,0);
     }
     setup_signals();
