@@ -265,7 +265,7 @@ static void send_drop_party_ack(SIG_ENTITY *sig,unsigned long call_ref,
 }
 
 
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 
 static void send_conn_avail(SOCKET *sock)
 {
@@ -370,7 +370,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    send_kernel(sock->id,0,as_okay,0,&sock->pvc,NULL,&sock->local,
 	      &sock->sap,&sock->qos);
 	    new_state(sock,ss_connected);
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 	    sock->owner = 1;
 #endif
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1,(struct sockaddr *)
@@ -391,7 +391,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    send_kernel(sock->id,0,as_okay,0,NULL,NULL,&sock->local,&sock->sap,
 	      NULL);
 	    new_state(sock,ss_connected);
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 	    sock->owner = 0;
 #endif
 	    if (atm2text(buffer,MAX_ATM_ADDR_LEN+1, (struct sockaddr *)
@@ -425,12 +425,12 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 		    STOP_TIMER(sock);
 		    free_sock(sock);
 		    return;
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		case ss_mod_req:
 #endif
 		    STOP_TIMER(sock);
 		    /* fall through */
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		case ss_mod_lcl:
 		case ss_mod_rcv:
 		case ss_mod_fin_ok:
@@ -440,7 +440,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 		case ss_connected:
 		    diag(COMPONENT,DIAG_INFO,"Passive close (CR 0x%06X)",
 		      sock->call_ref);
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		    if (timer_handler(sock->conn_timer) == on_T361)
 			STOP_TIMER(sock);
 #endif
@@ -481,8 +481,8 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 		    case ss_connecting:
 			set_error(sock,cause == ATM_CV_UNALLOC ?
 			  -EADDRNOTAVAIL : cause == ATM_CV_RES_UNAVAIL ||
-#if defined(UNI31) || defined(UNI40)
-			  cause == ATM_CV_UCR_UNAVAIL ||
+#if defined(UNI31) || defined(UNI40) || defined(DYNAMIC_UNI)
+			  cause == ATM_CV_UCR_UNAVAIL_NEW ||
 #endif
 			  cause == ATM_CV_NO_ROUTE_DEST ? -EHOSTUNREACH :
 			  cause == ATM_CV_NUM_CHANGED ? -EREMCHG :
@@ -502,12 +502,12 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 			STOP_TIMER(sock);
 			free_sock(sock);
 			return;
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		    case ss_mod_req:
 #endif
 			STOP_TIMER(sock);
 			/* fall through */
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 		    case ss_mod_lcl:
 		    case ss_mod_rcv:
 		    case ss_mod_fin_ok:
@@ -517,7 +517,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 		    case ss_connected:
 			diag(COMPONENT,DIAG_INFO,"Passive close (CR 0x%06X)",
 			  sock->call_ref);
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 			if (timer_handler(sock->conn_timer) == on_T361)
 			    STOP_TIMER(sock);
 #endif
@@ -557,11 +557,12 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	case ATM_MSG_NOTIFY:
 		/* silently ignore this junk */
 	    return;
-#ifdef Q2963_1
+#if defined(Q2963_1) || defined(DYNAMIC_UNI)
 /*
  * Buglet ahead: should actually test "call_state"
  */
 	case ATM_MSG_MODIFY_REQ:
+	    if (!(sock->sig->uni & S_Q2963_1)) goto _default;
 	    if (sock->state != ss_connected || sock->owner) break;
 	    sock->new_qos = sock->qos;
 	    if (q_present(&in_dsc,QF_fw_pcr_01))
@@ -573,6 +574,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    new_state(sock,ss_mod_rcv);
 	    return;
 	case ATM_MSG_MODIFY_ACK:
+	    if (!(sock->sig->uni & S_Q2963_1)) goto _default;
 	    if (sock->state != ss_mod_req) break;
 	    STOP_TIMER(sock);
 	    sock->qos = sock->new_qos;
@@ -582,6 +584,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    new_state(sock,ss_mod_fin_ok);
 	    return;
 	case ATM_MSG_MODIFY_REJ:
+	    if (!(sock->sig->uni & S_Q2963_1)) goto _default;
 	    if (sock->state != ss_mod_req) break;
 	    STOP_TIMER(sock);
 	    sock->error = -EAGAIN;
@@ -590,6 +593,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    new_state(sock,ss_mod_fin_fail);
 	    return;
 	case ATM_MSG_CONN_AVAIL:
+	    if (!(sock->sig->uni & S_Q2963_1)) goto _default;
 	    if (sock->state != ss_connected || !sock->owner) break;
 	    STOP_TIMER(sock);
 	    send_kernel(sock->id,0,as_modify,ATM_MF_SET,NULL,NULL,NULL,NULL,
@@ -598,6 +602,7 @@ static void uni_call(SOCKET *sock,unsigned char mid)
 	    return;
 #endif
 	default:
+	_default: /* jump here if we don't want to understand a message */
 	    diag(COMPONENT,DIAG_WARN,"Bad signaling message %d",mid);
 	    send_status(sock->sig,sock,0,ATM_CV_UNKNOWN_MSG_TYPE,mid);
 	    return;
